# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# INSTITUTO TECNOLOGICO DE COSTA RICA
# Escuela de Ingenieria en Construccion
# Construction Engineering School
# https://www.tec.ac.cr
# Eng. MSc. Maikel Mendez Morales
# Email: maikel.mendez@gmail.com; mamendez@itcr.ac.cr
# https://orcid.org/0000-0003-1919-141X
# https://www.scopus.com/authid/detail.uri?authorId=51665581300
# https://scholar.google.com/citations?user=JnmSVFYAAAAJ&hl=en
# https://www.youtube.com/c/maikelmendez
# https://twitter.com/MaikelMendezM
# https://github.com/maikelonu
# Skype: maikel.mendez
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

#-------------------------------------------------------------------------------------------------------------------
# INFO: This script is intended for the generation of spatially aggregated (mean) *. CSV statististics of 
# bias-corrected data by {R-Hyfo}, including empirical quantile mapping (EQM), gamma quantile mapping (GQM) 
# and linear scaling (SCALING) for the period 1981-1995 (15 years)
#-------------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------------
# MANUSCRIPT TITLE:
# "Performance Evaluation of Bias Correction Methods for Monthly Precipitation 
#  Projections over Costa Rica"
#-------------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------------
# INPUT FILES:

# netCDF format: all *.nc files included in the following lists

# Vector list for GPCC
# V.GPCC.EQM, V.GPCC.GQM, V.GPCC.SCALING,V.GPCC.RAW 

# Vector list for PRECIS
# V.PRECIS.EQM, V.PRECIS.GQM, V.PRECIS.SCALING, V.PRECIS.RAW 

# Vector list for CanESM2
# V.CanESM2.EQM, V.CanESM2.GQM, V.CanESM2.SCALING, V.CanESM2.RAW, 

# Vector list for MIROC
# V.MIROC.EQM, V.MIROC.GQM, V.MIROC.SCALING, V.MIROC.RAW 

# Vector list for EARTH
# V.EARTH.EQM, V.EARTH.GQM, V.EARTH.SCALING, V.EARTH.RAW 

# Vector list for HadGEM2
# V.HadGEM2.EQM, V.HadGEM2.GQM, V.HadGEM2.SCALING, V.HadGEM2.RAW 

# Vector list for ENSAMBLE
#V.ENSAMBLE.EQM, V.ENSAMBLE.GQM, V.ENSAMBLE.SCALING, V.ENSAMBLE.RAW 
#-------------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------------
# MANUSCRIPT FIGURES:
# None
#-------------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------------
# OUTPUT FILES:
# BC_R_HYFO.csv
#-------------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------------
# Workspace is cleared
#rm(list = ls())

# CRAN libraries are loaded
require(raster)
require(automap)
require(car)
require(chron)
require(compare)
require(DescTools)
require(dplyr)
require(fitdistrplus)
require(foreach)
require(forecast)
require(ggmap)
require(ggplot2)
require(gstat)
require(hyfo)
require(intamap)
require(KScorrect)
require(lattice)
require(lubridate)
require(maps)
require(maptools)
require(MASS)
require(ncdf4)
require(ncdf4.helpers)
require(pastecs)
require(PCICt)
require(plyr)
require(psych)
require(RColorBrewer)
require(rcompanion)
require(readr)
require(readxl)
require(reshape)
require(reshape2)
require(rgdal)
require(rgeos)
require(RNetCDF)
require(scales)
require(sp)
require(spatialEco)
require(tibble)
require(tidyr)
require(tseries)
require(viridis)
require(weathermetrics)

# Working directory is defined
setwd("/mnt/BACKUP/R_ITC/XLSX")  # UBUNTU-LINUX
setwd("C:/DATOS/R_ITC/XLSX")     # MS-WINDOWS

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: PRECIS georeference is loaded
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

# The blank map is transformed into SpatialGridDataFrame
raster.precis <- read.asciigrid("mod_region.asc", as.image = FALSE)

# The map is transformed into RasterLayer
raster.precis <- raster(raster.precis)

# CRTM05 projection for CR is created
CRTM05 <- CRS("+proj=tmerc +lat_0=0 +lon_0=-84 +k=0.9999 +x_0=500000 +y_0=0 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs")

# CRTM05 projection is assigned to rasterbrick object
crs(raster.precis) <- CRTM05

# Object attributes are requested
# It can be seen that spa.res is different !!!!
image(raster.precis, col = topo.colors(64))

# A simple plot is requested
plot(raster.precis)

# A summery is requested
summary(raster.precis)

# A dummy raster is created
raster.precis.dummy <- raster.precis

# Dummy NA values are introduced
values(raster.precis.dummy) <- NA

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Statistits of Bias-Correction by {R-Hyfo} EQM, GQM, SCALING
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

# Vector list for GPCC
V.GPCC.EQM <- c("GPCC_1981_1995_EQM_BC_JAN.nc","GPCC_1981_1995_EQM_BC_FEB.nc","GPCC_1981_1995_EQM_BC_MAR.nc","GPCC_1981_1995_EQM_BC_APR.nc","GPCC_1981_1995_EQM_BC_MAY.nc","GPCC_1981_1995_EQM_BC_JUN.nc","GPCC_1981_1995_EQM_BC_JUL.nc","GPCC_1981_1995_EQM_BC_AUG.nc","GPCC_1981_1995_EQM_BC_SEP.nc","GPCC_1981_1995_EQM_BC_OCT.nc","GPCC_1981_1995_EQM_BC_NOV.nc","GPCC_1981_1995_EQM_BC_DEC.nc","GPCC", "EQM")
V.GPCC.GQM <- c("GPCC_1981_1995_GQM_BC_JAN.nc","GPCC_1981_1995_GQM_BC_FEB.nc","GPCC_1981_1995_GQM_BC_MAR.nc","GPCC_1981_1995_GQM_BC_APR.nc","GPCC_1981_1995_GQM_BC_MAY.nc","GPCC_1981_1995_GQM_BC_JUN.nc","GPCC_1981_1995_GQM_BC_JUL.nc","GPCC_1981_1995_GQM_BC_AUG.nc","GPCC_1981_1995_GQM_BC_SEP.nc","GPCC_1981_1995_GQM_BC_OCT.nc","GPCC_1981_1995_GQM_BC_NOV.nc","GPCC_1981_1995_GQM_BC_DEC.nc","GPCC", "GQM")
V.GPCC.SCALING <- c("GPCC_1981_1995_SCALING_BC_JAN.nc","GPCC_1981_1995_SCALING_BC_FEB.nc","GPCC_1981_1995_SCALING_BC_MAR.nc","GPCC_1981_1995_SCALING_BC_APR.nc","GPCC_1981_1995_SCALING_BC_MAY.nc","GPCC_1981_1995_SCALING_BC_JUN.nc","GPCC_1981_1995_SCALING_BC_JUL.nc","GPCC_1981_1995_SCALING_BC_AUG.nc","GPCC_1981_1995_SCALING_BC_SEP.nc","GPCC_1981_1995_SCALING_BC_OCT.nc","GPCC_1981_1995_SCALING_BC_NOV.nc","GPCC_1981_1995_SCALING_BC_DEC.nc","GPCC", "SCALING")
V.GPCC.RAW <- c("GPCC_1981_1995_RAW_BC_JAN.nc","GPCC_1981_1995_RAW_BC_FEB.nc","GPCC_1981_1995_RAW_BC_MAR.nc","GPCC_1981_1995_RAW_BC_APR.nc","GPCC_1981_1995_RAW_BC_MAY.nc","GPCC_1981_1995_RAW_BC_JUN.nc","GPCC_1981_1995_RAW_BC_JUL.nc","GPCC_1981_1995_RAW_BC_AUG.nc","GPCC_1981_1995_RAW_BC_SEP.nc","GPCC_1981_1995_RAW_BC_OCT.nc","GPCC_1981_1995_RAW_BC_NOV.nc","GPCC_1981_1995_RAW_BC_DEC.nc","GPCC", "RAW")

# Vector list for PRECIS
V.PRECIS.EQM <- c("PRECIS_1981_1995_EQM_BC_JAN.nc","PRECIS_1981_1995_EQM_BC_FEB.nc","PRECIS_1981_1995_EQM_BC_MAR.nc","PRECIS_1981_1995_EQM_BC_APR.nc","PRECIS_1981_1995_EQM_BC_MAY.nc","PRECIS_1981_1995_EQM_BC_JUN.nc","PRECIS_1981_1995_EQM_BC_JUL.nc","PRECIS_1981_1995_EQM_BC_AUG.nc","PRECIS_1981_1995_EQM_BC_SEP.nc","PRECIS_1981_1995_EQM_BC_OCT.nc","PRECIS_1981_1995_EQM_BC_NOV.nc","PRECIS_1981_1995_EQM_BC_DEC.nc","PRECIS", "EQM")
V.PRECIS.GQM <- c("PRECIS_1981_1995_GQM_BC_JAN.nc","PRECIS_1981_1995_GQM_BC_FEB.nc","PRECIS_1981_1995_GQM_BC_MAR.nc","PRECIS_1981_1995_GQM_BC_APR.nc","PRECIS_1981_1995_GQM_BC_MAY.nc","PRECIS_1981_1995_GQM_BC_JUN.nc","PRECIS_1981_1995_GQM_BC_JUL.nc","PRECIS_1981_1995_GQM_BC_AUG.nc","PRECIS_1981_1995_GQM_BC_SEP.nc","PRECIS_1981_1995_GQM_BC_OCT.nc","PRECIS_1981_1995_GQM_BC_NOV.nc","PRECIS_1981_1995_GQM_BC_DEC.nc","PRECIS", "GQM")
V.PRECIS.SCALING <- c("PRECIS_1981_1995_SCALING_BC_JAN.nc","PRECIS_1981_1995_SCALING_BC_FEB.nc","PRECIS_1981_1995_SCALING_BC_MAR.nc","PRECIS_1981_1995_SCALING_BC_APR.nc","PRECIS_1981_1995_SCALING_BC_MAY.nc","PRECIS_1981_1995_SCALING_BC_JUN.nc","PRECIS_1981_1995_SCALING_BC_JUL.nc","PRECIS_1981_1995_SCALING_BC_AUG.nc","PRECIS_1981_1995_SCALING_BC_SEP.nc","PRECIS_1981_1995_SCALING_BC_OCT.nc","PRECIS_1981_1995_SCALING_BC_NOV.nc","PRECIS_1981_1995_SCALING_BC_DEC.nc","PRECIS", "SCALING")
V.PRECIS.RAW <- c("PRECIS_1981_1995_RAW_BC_JAN.nc","PRECIS_1981_1995_RAW_BC_FEB.nc","PRECIS_1981_1995_RAW_BC_MAR.nc","PRECIS_1981_1995_RAW_BC_APR.nc","PRECIS_1981_1995_RAW_BC_MAY.nc","PRECIS_1981_1995_RAW_BC_JUN.nc","PRECIS_1981_1995_RAW_BC_JUL.nc","PRECIS_1981_1995_RAW_BC_AUG.nc","PRECIS_1981_1995_RAW_BC_SEP.nc","PRECIS_1981_1995_RAW_BC_OCT.nc","PRECIS_1981_1995_RAW_BC_NOV.nc","PRECIS_1981_1995_RAW_BC_DEC.nc","PRECIS", "RAW")

# Vector list for CanESM2
V.CanESM2.EQM <- c("CanESM2_1981_1995_EQM.BC_JAN.nc","CanESM2_1981_1995_EQM.BC_FEB.nc","CanESM2_1981_1995_EQM.BC_MAR.nc","CanESM2_1981_1995_EQM.BC_APR.nc","CanESM2_1981_1995_EQM.BC_MAY.nc","CanESM2_1981_1995_EQM.BC_JUN.nc","CanESM2_1981_1995_EQM.BC_JUL.nc","CanESM2_1981_1995_EQM.BC_AUG.nc","CanESM2_1981_1995_EQM.BC_SEP.nc","CanESM2_1981_1995_EQM.BC_OCT.nc","CanESM2_1981_1995_EQM.BC_NOV.nc","CanESM2_1981_1995_EQM.BC_DEC.nc","CanESM2", "EQM")
V.CanESM2.GQM <- c("CanESM2_1981_1995_GQM.BC_JAN.nc","CanESM2_1981_1995_GQM.BC_FEB.nc","CanESM2_1981_1995_GQM.BC_MAR.nc","CanESM2_1981_1995_GQM.BC_APR.nc","CanESM2_1981_1995_GQM.BC_MAY.nc","CanESM2_1981_1995_GQM.BC_JUN.nc","CanESM2_1981_1995_GQM.BC_JUL.nc","CanESM2_1981_1995_GQM.BC_AUG.nc","CanESM2_1981_1995_GQM.BC_SEP.nc","CanESM2_1981_1995_GQM.BC_OCT.nc","CanESM2_1981_1995_GQM.BC_NOV.nc","CanESM2_1981_1995_GQM.BC_DEC.nc","CanESM2", "GQM")
V.CanESM2.SCALING <- c("CanESM2_1981_1995_SCALING.BC_JAN.nc","CanESM2_1981_1995_SCALING.BC_FEB.nc","CanESM2_1981_1995_SCALING.BC_MAR.nc","CanESM2_1981_1995_SCALING.BC_APR.nc","CanESM2_1981_1995_SCALING.BC_MAY.nc","CanESM2_1981_1995_SCALING.BC_JUN.nc","CanESM2_1981_1995_SCALING.BC_JUL.nc","CanESM2_1981_1995_SCALING.BC_AUG.nc","CanESM2_1981_1995_SCALING.BC_SEP.nc","CanESM2_1981_1995_SCALING.BC_OCT.nc","CanESM2_1981_1995_SCALING.BC_NOV.nc","CanESM2_1981_1995_SCALING.BC_DEC.nc","CanESM2", "SCALING")
V.CanESM2.RAW <- c("CanESM2_1981_1995_RAW.BC_JAN.nc","CanESM2_1981_1995_RAW.BC_FEB.nc","CanESM2_1981_1995_RAW.BC_MAR.nc","CanESM2_1981_1995_RAW.BC_APR.nc","CanESM2_1981_1995_RAW.BC_MAY.nc","CanESM2_1981_1995_RAW.BC_JUN.nc","CanESM2_1981_1995_RAW.BC_JUL.nc","CanESM2_1981_1995_RAW.BC_AUG.nc","CanESM2_1981_1995_RAW.BC_SEP.nc","CanESM2_1981_1995_RAW.BC_OCT.nc","CanESM2_1981_1995_RAW.BC_NOV.nc","CanESM2_1981_1995_RAW.BC_DEC.nc","CanESM2", "RAW")

# Vector list for MIROC
V.MIROC.EQM <- c("MIROC_1981_1995_EQM_BC_JAN.nc","MIROC_1981_1995_EQM_BC_FEB.nc","MIROC_1981_1995_EQM_BC_MAR.nc","MIROC_1981_1995_EQM_BC_APR.nc","MIROC_1981_1995_EQM_BC_MAY.nc","MIROC_1981_1995_EQM_BC_JUN.nc","MIROC_1981_1995_EQM_BC_JUL.nc","MIROC_1981_1995_EQM_BC_AUG.nc","MIROC_1981_1995_EQM_BC_SEP.nc","MIROC_1981_1995_EQM_BC_OCT.nc","MIROC_1981_1995_EQM_BC_NOV.nc","MIROC_1981_1995_EQM_BC_DEC.nc","MIROC", "EQM")
V.MIROC.GQM <- c("MIROC_1981_1995_GQM_BC_JAN.nc","MIROC_1981_1995_GQM_BC_FEB.nc","MIROC_1981_1995_GQM_BC_MAR.nc","MIROC_1981_1995_GQM_BC_APR.nc","MIROC_1981_1995_GQM_BC_MAY.nc","MIROC_1981_1995_GQM_BC_JUN.nc","MIROC_1981_1995_GQM_BC_JUL.nc","MIROC_1981_1995_GQM_BC_AUG.nc","MIROC_1981_1995_GQM_BC_SEP.nc","MIROC_1981_1995_GQM_BC_OCT.nc","MIROC_1981_1995_GQM_BC_NOV.nc","MIROC_1981_1995_GQM_BC_DEC.nc","MIROC", "GQM")
V.MIROC.SCALING <- c("MIROC_1981_1995_SCALING_BC_JAN.nc","MIROC_1981_1995_SCALING_BC_FEB.nc","MIROC_1981_1995_SCALING_BC_MAR.nc","MIROC_1981_1995_SCALING_BC_APR.nc","MIROC_1981_1995_SCALING_BC_MAY.nc","MIROC_1981_1995_SCALING_BC_JUN.nc","MIROC_1981_1995_SCALING_BC_JUL.nc","MIROC_1981_1995_SCALING_BC_AUG.nc","MIROC_1981_1995_SCALING_BC_SEP.nc","MIROC_1981_1995_SCALING_BC_OCT.nc","MIROC_1981_1995_SCALING_BC_NOV.nc","MIROC_1981_1995_SCALING_BC_DEC.nc","MIROC", "SCALING")
V.MIROC.RAW <- c("MIROC_1981_1995_RAW_BC_JAN.nc","MIROC_1981_1995_RAW_BC_FEB.nc","MIROC_1981_1995_RAW_BC_MAR.nc","MIROC_1981_1995_RAW_BC_APR.nc","MIROC_1981_1995_RAW_BC_MAY.nc","MIROC_1981_1995_RAW_BC_JUN.nc","MIROC_1981_1995_RAW_BC_JUL.nc","MIROC_1981_1995_RAW_BC_AUG.nc","MIROC_1981_1995_RAW_BC_SEP.nc","MIROC_1981_1995_RAW_BC_OCT.nc","MIROC_1981_1995_RAW_BC_NOV.nc","MIROC_1981_1995_RAW_BC_DEC.nc","MIROC", "RAW")

# Vector list for EARTH
V.EARTH.EQM <- c("EARTH_1981_1995_EQM_BC_JAN.nc","EARTH_1981_1995_EQM_BC_FEB.nc","EARTH_1981_1995_EQM_BC_MAR.nc","EARTH_1981_1995_EQM_BC_APR.nc","EARTH_1981_1995_EQM_BC_MAY.nc","EARTH_1981_1995_EQM_BC_JUN.nc","EARTH_1981_1995_EQM_BC_JUL.nc","EARTH_1981_1995_EQM_BC_AUG.nc","EARTH_1981_1995_EQM_BC_SEP.nc","EARTH_1981_1995_EQM_BC_OCT.nc","EARTH_1981_1995_EQM_BC_NOV.nc","EARTH_1981_1995_EQM_BC_DEC.nc","EARTH", "EQM")
V.EARTH.GQM <- c("EARTH_1981_1995_GQM_BC_JAN.nc","EARTH_1981_1995_GQM_BC_FEB.nc","EARTH_1981_1995_GQM_BC_MAR.nc","EARTH_1981_1995_GQM_BC_APR.nc","EARTH_1981_1995_GQM_BC_MAY.nc","EARTH_1981_1995_GQM_BC_JUN.nc","EARTH_1981_1995_GQM_BC_JUL.nc","EARTH_1981_1995_GQM_BC_AUG.nc","EARTH_1981_1995_GQM_BC_SEP.nc","EARTH_1981_1995_GQM_BC_OCT.nc","EARTH_1981_1995_GQM_BC_NOV.nc","EARTH_1981_1995_GQM_BC_DEC.nc","EARTH", "GQM")
V.EARTH.SCALING <- c("EARTH_1981_1995_SCALING_BC_JAN.nc","EARTH_1981_1995_SCALING_BC_FEB.nc","EARTH_1981_1995_SCALING_BC_MAR.nc","EARTH_1981_1995_SCALING_BC_APR.nc","EARTH_1981_1995_SCALING_BC_MAY.nc","EARTH_1981_1995_SCALING_BC_JUN.nc","EARTH_1981_1995_SCALING_BC_JUL.nc","EARTH_1981_1995_SCALING_BC_AUG.nc","EARTH_1981_1995_SCALING_BC_SEP.nc","EARTH_1981_1995_SCALING_BC_OCT.nc","EARTH_1981_1995_SCALING_BC_NOV.nc","EARTH_1981_1995_SCALING_BC_DEC.nc","EARTH", "SCALING")
V.EARTH.RAW <- c("EARTH_1981_1995_RAW_BC_JAN.nc","EARTH_1981_1995_RAW_BC_FEB.nc","EARTH_1981_1995_RAW_BC_MAR.nc","EARTH_1981_1995_RAW_BC_APR.nc","EARTH_1981_1995_RAW_BC_MAY.nc","EARTH_1981_1995_RAW_BC_JUN.nc","EARTH_1981_1995_RAW_BC_JUL.nc","EARTH_1981_1995_RAW_BC_AUG.nc","EARTH_1981_1995_RAW_BC_SEP.nc","EARTH_1981_1995_RAW_BC_OCT.nc","EARTH_1981_1995_RAW_BC_NOV.nc","EARTH_1981_1995_RAW_BC_DEC.nc","EARTH", "RAW")

# Vector list for HadGEM2
V.HadGEM2.EQM <- c("HadGEM2_1981_1995_EQM_BC_JAN.nc","HadGEM2_1981_1995_EQM_BC_FEB.nc","HadGEM2_1981_1995_EQM_BC_MAR.nc","HadGEM2_1981_1995_EQM_BC_APR.nc","HadGEM2_1981_1995_EQM_BC_MAY.nc","HadGEM2_1981_1995_EQM_BC_JUN.nc","HadGEM2_1981_1995_EQM_BC_JUL.nc","HadGEM2_1981_1995_EQM_BC_AUG.nc","HadGEM2_1981_1995_EQM_BC_SEP.nc","HadGEM2_1981_1995_EQM_BC_OCT.nc","HadGEM2_1981_1995_EQM_BC_NOV.nc","HadGEM2_1981_1995_EQM_BC_DEC.nc","HadGEM2", "EQM")
V.HadGEM2.GQM <- c("HadGEM2_1981_1995_GQM_BC_JAN.nc","HadGEM2_1981_1995_GQM_BC_FEB.nc","HadGEM2_1981_1995_GQM_BC_MAR.nc","HadGEM2_1981_1995_GQM_BC_APR.nc","HadGEM2_1981_1995_GQM_BC_MAY.nc","HadGEM2_1981_1995_GQM_BC_JUN.nc","HadGEM2_1981_1995_GQM_BC_JUL.nc","HadGEM2_1981_1995_GQM_BC_AUG.nc","HadGEM2_1981_1995_GQM_BC_SEP.nc","HadGEM2_1981_1995_GQM_BC_OCT.nc","HadGEM2_1981_1995_GQM_BC_NOV.nc","HadGEM2_1981_1995_GQM_BC_DEC.nc","HadGEM2", "GQM")
V.HadGEM2.SCALING <- c("HadGEM2_1981_1995_SCALING_BC_JAN.nc","HadGEM2_1981_1995_SCALING_BC_FEB.nc","HadGEM2_1981_1995_SCALING_BC_MAR.nc","HadGEM2_1981_1995_SCALING_BC_APR.nc","HadGEM2_1981_1995_SCALING_BC_MAY.nc","HadGEM2_1981_1995_SCALING_BC_JUN.nc","HadGEM2_1981_1995_SCALING_BC_JUL.nc","HadGEM2_1981_1995_SCALING_BC_AUG.nc","HadGEM2_1981_1995_SCALING_BC_SEP.nc","HadGEM2_1981_1995_SCALING_BC_OCT.nc","HadGEM2_1981_1995_SCALING_BC_NOV.nc","HadGEM2_1981_1995_SCALING_BC_DEC.nc","HadGEM2", "SCALING")
V.HadGEM2.RAW <- c("HadGEM2_1981_1995_RAW_BC_JAN.nc","HadGEM2_1981_1995_RAW_BC_FEB.nc","HadGEM2_1981_1995_RAW_BC_MAR.nc","HadGEM2_1981_1995_RAW_BC_APR.nc","HadGEM2_1981_1995_RAW_BC_MAY.nc","HadGEM2_1981_1995_RAW_BC_JUN.nc","HadGEM2_1981_1995_RAW_BC_JUL.nc","HadGEM2_1981_1995_RAW_BC_AUG.nc","HadGEM2_1981_1995_RAW_BC_SEP.nc","HadGEM2_1981_1995_RAW_BC_OCT.nc","HadGEM2_1981_1995_RAW_BC_NOV.nc","HadGEM2_1981_1995_RAW_BC_DEC.nc","HadGEM2", "RAW")

# Vector list for MPI
V.MPI.EQM <- c("MPI_1981_1995_EQM_BC_JAN.nc","MPI_1981_1995_EQM_BC_FEB.nc","MPI_1981_1995_EQM_BC_MAR.nc","MPI_1981_1995_EQM_BC_APR.nc","MPI_1981_1995_EQM_BC_MAY.nc","MPI_1981_1995_EQM_BC_JUN.nc","MPI_1981_1995_EQM_BC_JUL.nc","MPI_1981_1995_EQM_BC_AUG.nc","MPI_1981_1995_EQM_BC_SEP.nc","MPI_1981_1995_EQM_BC_OCT.nc","MPI_1981_1995_EQM_BC_NOV.nc","MPI_1981_1995_EQM_BC_DEC.nc","MPI", "EQM")
V.MPI.GQM <- c("MPI_1981_1995_GQM_BC_JAN.nc","MPI_1981_1995_GQM_BC_FEB.nc","MPI_1981_1995_GQM_BC_MAR.nc","MPI_1981_1995_GQM_BC_APR.nc","MPI_1981_1995_GQM_BC_MAY.nc","MPI_1981_1995_GQM_BC_JUN.nc","MPI_1981_1995_GQM_BC_JUL.nc","MPI_1981_1995_GQM_BC_AUG.nc","MPI_1981_1995_GQM_BC_SEP.nc","MPI_1981_1995_GQM_BC_OCT.nc","MPI_1981_1995_GQM_BC_NOV.nc","MPI_1981_1995_GQM_BC_DEC.nc","MPI", "GQM")
V.MPI.SCALING <- c("MPI_1981_1995_SCALING_BC_JAN.nc","MPI_1981_1995_SCALING_BC_FEB.nc","MPI_1981_1995_SCALING_BC_MAR.nc","MPI_1981_1995_SCALING_BC_APR.nc","MPI_1981_1995_SCALING_BC_MAY.nc","MPI_1981_1995_SCALING_BC_JUN.nc","MPI_1981_1995_SCALING_BC_JUL.nc","MPI_1981_1995_SCALING_BC_AUG.nc","MPI_1981_1995_SCALING_BC_SEP.nc","MPI_1981_1995_SCALING_BC_OCT.nc","MPI_1981_1995_SCALING_BC_NOV.nc","MPI_1981_1995_SCALING_BC_DEC.nc","MPI", "SCALING")
V.MPI.RAW <- c("MPI_1981_1995_RAW_BC_JAN.nc","MPI_1981_1995_RAW_BC_FEB.nc","MPI_1981_1995_RAW_BC_MAR.nc","MPI_1981_1995_RAW_BC_APR.nc","MPI_1981_1995_RAW_BC_MAY.nc","MPI_1981_1995_RAW_BC_JUN.nc","MPI_1981_1995_RAW_BC_JUL.nc","MPI_1981_1995_RAW_BC_AUG.nc","MPI_1981_1995_RAW_BC_SEP.nc","MPI_1981_1995_RAW_BC_OCT.nc","MPI_1981_1995_RAW_BC_NOV.nc","MPI_1981_1995_RAW_BC_DEC.nc","MPI", "RAW")

# Vector list for ENSAMBLE
V.ENSAMBLE.EQM <- c("ENSAMBLE_1981_1995_EQM_BC_JAN.nc","ENSAMBLE_1981_1995_EQM_BC_FEB.nc","ENSAMBLE_1981_1995_EQM_BC_MAR.nc","ENSAMBLE_1981_1995_EQM_BC_APR.nc","ENSAMBLE_1981_1995_EQM_BC_MAY.nc","ENSAMBLE_1981_1995_EQM_BC_JUN.nc","ENSAMBLE_1981_1995_EQM_BC_JUL.nc","ENSAMBLE_1981_1995_EQM_BC_AUG.nc","ENSAMBLE_1981_1995_EQM_BC_SEP.nc","ENSAMBLE_1981_1995_EQM_BC_OCT.nc","ENSAMBLE_1981_1995_EQM_BC_NOV.nc","ENSAMBLE_1981_1995_EQM_BC_DEC.nc","ENSAMBLE", "EQM")
V.ENSAMBLE.GQM <- c("ENSAMBLE_1981_1995_GQM_BC_JAN.nc","ENSAMBLE_1981_1995_GQM_BC_FEB.nc","ENSAMBLE_1981_1995_GQM_BC_MAR.nc","ENSAMBLE_1981_1995_GQM_BC_APR.nc","ENSAMBLE_1981_1995_GQM_BC_MAY.nc","ENSAMBLE_1981_1995_GQM_BC_JUN.nc","ENSAMBLE_1981_1995_GQM_BC_JUL.nc","ENSAMBLE_1981_1995_GQM_BC_AUG.nc","ENSAMBLE_1981_1995_GQM_BC_SEP.nc","ENSAMBLE_1981_1995_GQM_BC_OCT.nc","ENSAMBLE_1981_1995_GQM_BC_NOV.nc","ENSAMBLE_1981_1995_GQM_BC_DEC.nc","ENSAMBLE", "GQM")
V.ENSAMBLE.SCALING <- c("ENSAMBLE_1981_1995_SCALING_BC_JAN.nc","ENSAMBLE_1981_1995_SCALING_BC_FEB.nc","ENSAMBLE_1981_1995_SCALING_BC_MAR.nc","ENSAMBLE_1981_1995_SCALING_BC_APR.nc","ENSAMBLE_1981_1995_SCALING_BC_MAY.nc","ENSAMBLE_1981_1995_SCALING_BC_JUN.nc","ENSAMBLE_1981_1995_SCALING_BC_JUL.nc","ENSAMBLE_1981_1995_SCALING_BC_AUG.nc","ENSAMBLE_1981_1995_SCALING_BC_SEP.nc","ENSAMBLE_1981_1995_SCALING_BC_OCT.nc","ENSAMBLE_1981_1995_SCALING_BC_NOV.nc","ENSAMBLE_1981_1995_SCALING_BC_DEC.nc","ENSAMBLE", "SCALING")
V.ENSAMBLE.RAW <- c("ENSAMBLE_1981_1995_RAW_BC_JAN.nc","ENSAMBLE_1981_1995_RAW_BC_FEB.nc","ENSAMBLE_1981_1995_RAW_BC_MAR.nc","ENSAMBLE_1981_1995_RAW_BC_APR.nc","ENSAMBLE_1981_1995_RAW_BC_MAY.nc","ENSAMBLE_1981_1995_RAW_BC_JUN.nc","ENSAMBLE_1981_1995_RAW_BC_JUL.nc","ENSAMBLE_1981_1995_RAW_BC_AUG.nc","ENSAMBLE_1981_1995_RAW_BC_SEP.nc","ENSAMBLE_1981_1995_RAW_BC_OCT.nc","ENSAMBLE_1981_1995_RAW_BC_NOV.nc","ENSAMBLE_1981_1995_RAW_BC_DEC.nc","ENSAMBLE", "RAW")

# A main-indexing list is created
main.list <- list(V.GPCC.EQM, V.GPCC.GQM, V.GPCC.SCALING, V.GPCC.RAW, 
                  V.PRECIS.EQM, V.PRECIS.GQM, V.PRECIS.SCALING, V.PRECIS.RAW,
                  V.CanESM2.EQM,V.CanESM2.GQM,V.CanESM2.SCALING,V.CanESM2.RAW,
                  V.MIROC.EQM,V.MIROC.GQM,V.MIROC.SCALING,V.MIROC.RAW,
                  V.EARTH.EQM,V.EARTH.GQM,V.EARTH.SCALING,V.EARTH.RAW,
                  V.HadGEM2.EQM,V.HadGEM2.GQM,V.HadGEM2.SCALING,V.HadGEM2.RAW,
                  V.MPI.EQM,V.MPI.GQM,V.MPI.SCALING,V.MPI.RAW,
                  V.ENSAMBLE.EQM,V.ENSAMBLE.GQM,V.ENSAMBLE.SCALING,V.ENSAMBLE.RAW)

# IMN 1981-1995 historical is loaded
hist.1981_1995.IMN <- stack("hist_1981_1995_IMN.nc")

# Spatial bias is calculated (modelled - observed!!)
l.observed <- hist.1981_1995.IMN

# CRTM05 projection is assigned to rasterbrick object
crs(l.observed) <- CRTM05

# A "levels" monthly vector is created for visualization puposes
v.select.name <- c("JAN", "FEB", "MAR","APR", "MAY", "JUN", "JUL","AUG", "SEP", "OCT", "NOV","DEC")

# A list consyainer is defined
list.export <- NULL
list.export <- list()

#----------------------------------------------------------------------------------------------------
# Raster ASC CR Climatic Regions Masking
#----------------------------------------------------------------------------------------------------
# Main loop is initiated
for (ii in 1:length(main.list)) {
  
  # Variable list is defined
  var.vector <- main.list[[ii]]
  
  # Historical PRECIS Rasterbrick object is imported from GeoTIFF format
  l.modelled.JAN <- stack(var.vector[1]) # BEWARE: CRTM05 !!!!
  l.modelled.FEB <- stack(var.vector[2]) # BEWARE: CRTM05 !!!!
  l.modelled.MAR <- stack(var.vector[3]) # BEWARE: CRTM05 !!!!
  l.modelled.APR <- stack(var.vector[4]) # BEWARE: CRTM05 !!!!
  l.modelled.MAY <- stack(var.vector[5]) # BEWARE: CRTM05 !!!!
  l.modelled.JUN <- stack(var.vector[6]) # BEWARE: CRTM05 !!!!
  l.modelled.JUL <- stack(var.vector[7]) # BEWARE: CRTM05 !!!!
  l.modelled.AUG <- stack(var.vector[8]) # BEWARE: CRTM05 !!!!
  l.modelled.SEP <- stack(var.vector[9]) # BEWARE: CRTM05 !!!!
  l.modelled.OCT <- stack(var.vector[10]) # BEWARE: CRTM05 !!!!
  l.modelled.NOV <- stack(var.vector[11]) # BEWARE: CRTM05 !!!!
  l.modelled.DEC <- stack(var.vector[12]) # BEWARE: CRTM05 !!!!
  
  # CRTM05 projection is assigned to rasterbrick object
  crs(l.modelled.JAN) <- CRTM05
  crs(l.modelled.FEB) <- CRTM05
  crs(l.modelled.MAR) <- CRTM05
  crs(l.modelled.APR) <- CRTM05
  crs(l.modelled.MAY) <- CRTM05
  crs(l.modelled.JUN) <- CRTM05
  crs(l.modelled.JUL) <- CRTM05
  crs(l.modelled.AUG) <- CRTM05
  crs(l.modelled.SEP) <- CRTM05
  crs(l.modelled.OCT) <- CRTM05
  crs(l.modelled.NOV) <- CRTM05
  crs(l.modelled.DEC) <- CRTM05
  
  # Monthly variables are loaded
  vss.JAN <- c(1,13,25,37,49,61,73,85,97,109,121,133,145,157,169)
  vss.FEB <- c(2,14,26,38,50,62,74,86,98,110,122,134,146,158,170)
  vss.MAR <- c(3,15,27,39,51,63,75,87,99,111,123,135,147,159,171)
  vss.APR <- c(4,16,28,40,52,64,76,88,100,112,124,136,148,160,172)
  vss.MAY <- c(5,17,29,41,53,65,77,89,101,113,125,137,149,161,173)
  vss.JUN <- c(6,18,30,42,54,66,78,90,102,114,126,138,150,162,174)
  vss.JUL <- c(7,19,31,43,55,67,79,91,103,115,127,139,151,163,175)
  vss.AUG <- c(8,20,32,44,56,68,80,92,104,116,128,140,152,164,176)
  vss.SEP <- c(9,21,33,45,57,69,81,93,105,117,129,141,153,165,177)
  vss.OCT <- c(10,22,34,46,58,70,82,94,106,118,130,142,154,166,178)
  vss.NOV <- c(11,23,35,47,59,71,83,95,107,119,131,143,155,167,179)
  vss.DEC <- c(12,24,36,48,60,72,84,96,108,120,132,144,156,168,180) 
  
  #-------------------------------------------------------------------------------------
  # SUBBLOCK: ASC CR Climatic Regions Masking
  #-------------------------------------------------------------------------------------
  
  # A list container is defined
  df.list <- NULL
  df.list <- list()
  
  #----------------------------------------------------------------------------------------------------
  # Climatic region coding
  #----------------------------------------------------------------------------------------------------
  # North_Pacific = 3
  # Central_Pacific = 2
  # South_Pacific = 5
  # Central = 4
  # North = 1
  # Caribbean = 6
  #----------------------------------------------------------------------------------------------------
  
  # Climatic region is selected
  CR <- c(1,2,3,4,5,6)
  
  # Secondary loop is initiated
  for (uu in CR) {
    
    df.CDFs <- NULL
    
    # Climatic-regions CR blank raster is loaded
    blank <- paste("mod_region.asc", sep = "/")
    
    # The blank map is transformed into SpatialGridDataFrame
    blank.grid <- read.asciigrid(blank, as.image = FALSE)
    
    # The map is transformed into RasterLayer
    blank.raster <- raster(blank.grid)
    
    # CRTM05 projection is assigned to rasterbrick object
    crs(blank.raster) <- CRTM05
    
    # A simple plot is requested
    # plot(blank.raster)
    
    # Main loop counter is printed
    print(paste0("Current SEQ out of 28: ", ii))
    
    # Climatic region is selected
    values(blank.raster)[values(blank.raster) != uu] <- NA
    
    # A simple plot is requested
    plot(blank.raster)
    
    # ------------------------------------------------------------------------------------------
    # Monthly observations vs. modelled outputs are masked and spatially aggregated (mean)
    # ------------------------------------------------------------------------------------------
    rt.mask.JAN.obs <- mask(l.observed[[vss.JAN]], blank.raster)
    rt.mask.JAN <- mask(l.modelled.JAN, blank.raster)
    delta_cell.JAN <- (cellStats(rt.mask.JAN, mean, na.rm = TRUE))
    delta_cell.JAN.obs <- (cellStats(rt.mask.JAN.obs, mean, na.rm = TRUE))
    
    rt.mask.FEB.obs <- mask(l.observed[[vss.FEB]], blank.raster)
    rt.mask.FEB <- mask(l.modelled.FEB, blank.raster)
    delta_cell.FEB <- (cellStats(rt.mask.FEB, mean, na.rm = TRUE))
    delta_cell.FEB.obs <- (cellStats(rt.mask.FEB.obs, mean, na.rm = TRUE))
    
    rt.mask.MAR.obs <- mask(l.observed[[vss.MAR]], blank.raster)
    rt.mask.MAR <- mask(l.modelled.MAR, blank.raster)
    delta_cell.MAR <- (cellStats(rt.mask.MAR, mean, na.rm = TRUE))
    delta_cell.MAR.obs <- (cellStats(rt.mask.MAR.obs, mean, na.rm = TRUE))
    
    rt.mask.APR.obs <- mask(l.observed[[vss.APR]], blank.raster)
    rt.mask.APR <- mask(l.modelled.APR, blank.raster)
    delta_cell.APR <- (cellStats(rt.mask.APR, mean, na.rm = TRUE))
    delta_cell.APR.obs <- (cellStats(rt.mask.APR.obs, mean, na.rm = TRUE))
    
    rt.mask.MAY.obs <- mask(l.observed[[vss.MAY]], blank.raster)
    rt.mask.MAY <- mask(l.modelled.MAY, blank.raster)
    delta_cell.MAY <- (cellStats(rt.mask.MAY, mean, na.rm = TRUE))
    delta_cell.MAY.obs <- (cellStats(rt.mask.MAY.obs, mean, na.rm = TRUE))
    
    rt.mask.JUN.obs <- mask(l.observed[[vss.JUN]], blank.raster)
    rt.mask.JUN <- mask(l.modelled.JUN, blank.raster)
    delta_cell.JUN <- (cellStats(rt.mask.JUN, mean, na.rm = TRUE))
    delta_cell.JUN.obs <- (cellStats(rt.mask.JUN.obs, mean, na.rm = TRUE))
    
    rt.mask.JUL.obs <- mask(l.observed[[vss.JUL]], blank.raster)
    rt.mask.JUL <- mask(l.modelled.JUL, blank.raster)
    delta_cell.JUL <- (cellStats(rt.mask.JUL, mean, na.rm = TRUE))
    delta_cell.JUL.obs <- (cellStats(rt.mask.JUL.obs, mean, na.rm = TRUE))
    
    rt.mask.AUG.obs <- mask(l.observed[[vss.AUG]], blank.raster)
    rt.mask.AUG <- mask(l.modelled.AUG, blank.raster)
    delta_cell.AUG <- (cellStats(rt.mask.AUG, mean, na.rm = TRUE))
    delta_cell.AUG.obs <- (cellStats(rt.mask.AUG.obs, mean, na.rm = TRUE))
    
    rt.mask.SEP.obs <- mask(l.observed[[vss.SEP]], blank.raster)
    rt.mask.SEP <- mask(l.modelled.SEP, blank.raster)
    delta_cell.SEP <- (cellStats(rt.mask.SEP, mean, na.rm = TRUE))
    delta_cell.SEP.obs <- (cellStats(rt.mask.SEP.obs, mean, na.rm = TRUE))
    
    rt.mask.OCT.obs <- mask(l.observed[[vss.OCT]], blank.raster)
    rt.mask.OCT <- mask(l.modelled.OCT, blank.raster)
    delta_cell.OCT <- (cellStats(rt.mask.OCT, mean, na.rm = TRUE))
    delta_cell.OCT.obs <- (cellStats(rt.mask.OCT.obs, mean, na.rm = TRUE))
    
    rt.mask.NOV.obs <- mask(l.observed[[vss.NOV]], blank.raster)
    rt.mask.NOV <- mask(l.modelled.NOV, blank.raster)
    delta_cell.NOV <- (cellStats(rt.mask.NOV, mean, na.rm = TRUE))
    delta_cell.NOV.obs <- (cellStats(rt.mask.NOV.obs, mean, na.rm = TRUE))
    
    rt.mask.DEC.obs <- mask(l.observed[[vss.DEC]], blank.raster)
    rt.mask.DEC <- mask(l.modelled.DEC, blank.raster)
    delta_cell.DEC <- (cellStats(rt.mask.DEC, mean, na.rm = TRUE))
    delta_cell.DEC.obs <- (cellStats(rt.mask.DEC.obs, mean, na.rm = TRUE))
    
    # An mean-spatially aggregated modelled vector is created
    vector.delta <- c(delta_cell.JAN,
                      delta_cell.FEB,
                      delta_cell.MAR,
                      delta_cell.APR,
                      delta_cell.MAY,
                      delta_cell.JUN,
                      delta_cell.JUL,
                      delta_cell.AUG,
                      delta_cell.SEP,
                      delta_cell.OCT,
                      delta_cell.NOV,
                      delta_cell.DEC)
    
    # An mean-spatially aggregated observed vector is created
    vector.delta.obs <- c(delta_cell.JAN.obs,
                          delta_cell.FEB.obs,
                          delta_cell.MAR.obs,
                          delta_cell.APR.obs,
                          delta_cell.MAY.obs,
                          delta_cell.JUN.obs,
                          delta_cell.JUL.obs,
                          delta_cell.AUG.obs,
                          delta_cell.SEP.obs,
                          delta_cell.OCT.obs,
                          delta_cell.NOV.obs,
                          delta_cell.DEC.obs)
    
    # A labelling vector is created
    vector.month <- c(rep(c("JAN"),15),
                      rep(c("FEB"),15),
                      rep(c("MAR"),15),
                      rep(c("APR"),15),
                      rep(c("MAY"),15),
                      rep(c("JUN"),15),
                      rep(c("JUL"),15),
                      rep(c("AUG"),15),
                      rep(c("SEP"),15),
                      rep(c("OCT"),15),
                      rep(c("NOV"),15),
                      rep(c("DEC"),15))
    
    # Vector names are deleted
    names(vector.delta) <- NULL
    
    # Variables are rounded to 3 decimals
    round(vector.delta, 3)
    
    # A labelling vector is created
    rep(v.select.name,14)
    
    # A compiled data.frame is created
    df.CDFs <- data.frame(rep(var.vector[13], length(vector.delta)), # Model
                          vector.delta,                              # Model_value
                          vector.delta.obs,                          # Obs_value
                          vector.month,                              # Month
                          rep(var.vector[14], length(vector.delta)), # BC_method
                          rep(uu, length(vector.delta)))             # Region
    
    # data.frame variables are renamed
    names(df.CDFs) <- c("Model", "Model_value", "Obs_value", "Month", "BC_method", "Region")
    
    # A list of data.frames is created
    df.list[[uu]] <- df.CDFs
    
  } # Secondary loop is closed
  
  # List container is dumped
  df.export.internal <- NULL
  
  # An internal list of data.frames is created
  df.export.internal <- ldply(df.list, data.frame)
  
  # An internal list is created
  list.export[[ii]] <- df.export.internal
  
} # Main loop is closed

# An external list of data.frames is created
df.export.def <- ldply(list.export, data.frame)

# data.frame is requested
View(df.export.def)

# A BC {R-Hyfo} EQM, GQM, SCALING *.CSV database is exported
write.csv(df.export.def, file = "BC_R_HYFO.csv")

#-------------------------------------------------------------------------------------------------------------------
# END of code
#-------------------------------------------------------------------------------------------------------------------